#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

typedef struct HidReport
{
    /*
     * +----\------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |Byte \ Bit |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
     * +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  0        |                 Light (L)                     |
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  1        |            UNUSED           | Btn | Light (H) |
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     *
     */
    uint8_t  button1:1;
    uint8_t  button2:1;
    uint8_t  button3:1;
    uint8_t  button4:1;
    uint8_t  button5:1;
    uint8_t  button6:1;
    uint8_t  button7:1;
    uint8_t  button8:1;
} HidReport_t;

static HidReport_t reportBuffer;
static uchar idleRate; // Unused

/////////////////////////////////////////////////////////////////////
// USB report descriptor, generated by HID Descriptor Tool from
//    http://www.usb.org/developers/hidpage/dt2_4.zip
/////////////////////////////////////////////////////////////////////
PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x05,                    // USAGE (Game Pad)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0x05, 0x09,                    //   USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x08,                    //   USAGE_MAXIMUM (Button 8)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0xc0                           // END_COLLECTION
};

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void buildReport()
{
/*
    ((PINB & (1<<PC4)) == 0);
    ((PINB & (1<<PC2)) == 0);
    ((PINB & (1<<PC0)) == 0);
    ((PINB & (1<<PC1)) == 0);

    ((PINC & (1<<PC0)) == 1);
    !((PINC & (1<<PC2)) == 0);
    !((PINC & (1<<PC4)) == 0);
    ((PINC & (1<<PC3)) == 0);
*/
	reportBuffer.button1 = ((PINB & (1<<PC4)) == 0) && ((PINC & (1<<PC0)) == 1) && (!((PINB & (1<<PC0)) == 0) && ((PINC & (1<<PC4)) == 0)); //UP
	reportBuffer.button2 = ((PINB & (1<<PC2)) == 0) && ((PINC & (1<<PC2)) == 0) && ((PINB & (1<<PC2)) == 0) && ((PINC & (1<<PC2)) == 0)  //LEFT
	reportBuffer.button3 = !((PINB & (1<<PC2)) == 0) && !((PINC & (1<<PC2)) == 0) && !((PINB & (1<<PC2)) == 0) && !((PINC & (1<<PC2)) == 0)  //RIGHT
	reportBuffer.button4 = ((PINB & (1<<PC0)) == 0) && !((PINC & (1<<PC4)) == 0) && (!((PINB & (1<<PC4)) == 0) && !((PINC & (1<<PC0)) == 1)); //DOWN
	reportBuffer.button5 = ((PINB & (1<<PC4)) == 0) && ((PINB & (1<<PC2)) == 0) && ((PINB & (1<<PC0)) == 0) && ((PINC & (1<<PC0)) == 1) && !((PINC & (1<<PC2)) == 0) && !((PINC & (1<<PC4)) == 0); //BUMP
	reportBuffer.button6 = 0;
	reportBuffer.button7 = 0;
	reportBuffer.button8 = 0;
}

///////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    if(rq->bRequest == USBRQ_HID_GET_REPORT)
    {
        usbMsgPtr = (void *)&reportBuffer;
        return sizeof(reportBuffer);
    }
    else if (rq->bRequest == USBRQ_HID_GET_IDLE)
    {
        usbMsgPtr = &idleRate;
        return 1;
    }
    else if (rq->bRequest == USBRQ_HID_SET_IDLE)
    {
        idleRate = rq->wValue.bytes[1];
    }

    return 0;
}

///////////////////////////////////////////////////////////
int main(void)
{
    DDRB &= ~(1<<PC0);
    DDRB &= ~(1<<PC2);
    DDRB &= ~(1<<PC4);
    PORTB = (1<<PC0)|(1<<PC2)|(1<<PC4); 
    //DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2); // PC0..PC2 = Output, PC3..PC5 = Input
    DDRC &= ~(1<<PC4);
    DDRC &= ~(1<<PC2);
    DDRC &= ~(1<<PC0);
    PORTC = (1<<PC4)|(1<<PC2)|(1<<PC0);  // Enable pull-up at PC3


    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        /* always have report ready */
        buildReport();

        usbPoll();

        /* let the host know without asking */
        if (usbInterruptIsReady())
        {
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
